@@ -6,10 +6,36 @@ use XSLoader;
 
 use Exporter 5.57 'import';
 
-our $VERSION     = '0.008';
+our $VERSION     = '0.010';
 our %EXPORT_TAGS = ( 'all' => [qw<
-    is_scalarref is_arrayref is_hashref is_coderef is_regexpref
-    is_globref is_formatref is_ioref is_refref is_ref
+    is_ref
+    is_scalarref
+    is_arrayref
+    is_hashref
+    is_coderef
+    is_regexpref
+    is_globref
+    is_formatref
+    is_ioref
+    is_refref
+
+    is_plain_ref
+    is_plain_scalarref
+    is_plain_arrayref
+    is_plain_hashref
+    is_plain_coderef
+    is_plain_globref
+    is_plain_formatref
+    is_plain_refref
+
+    is_blessed_ref
+    is_blessed_scalarref
+    is_blessed_arrayref
+    is_blessed_hashref
+    is_blessed_coderef
+    is_blessed_globref
+    is_blessed_formatref
+    is_blessed_refref
 >] );
 our @EXPORT      = ();
 our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
@@ -30,7 +56,7 @@ Ref::Util - Utility functions for checking references
 
 =head1 VERSION
 
-0.008
+0.010
 
 =head1 DESCRIPTION
 
@@ -98,6 +124,13 @@ string:
 
 =item * Supports blessed variables
 
+B<Note:> In future versions, the idea is to make the default functions
+use the B<plain> variation, which means explicitly non-blessed references.
+If you want to explicitly check for B<blessed> references, you should use
+the C<is_blessed_*> functions. There will be an C<is_any_*> variation
+which will act like the current main functions - not caring whether it's
+blessed or not.
+
 When calling C<ref>, you receive either the reference type (B<SCALAR>,
 B<ARRAY>, B<HASH>, etc.) or the package it's blessed into.
 
@@ -111,6 +144,17 @@ so even if it's blessed, you know what type of variable is blessed.
     my $foo = bless {}, 'PKG';
     is_hashref($foo); # works
 
+On the other hand, in some situations it might be better to specifically
+exclude blessed references. The rationale for that might be that merely
+because some object happens to be implemented using a hash doesn't mean it's
+necessarily correct to treat it as a hash. For these situations, you can use
+C<is_plain_hashref> and friends, which have the same performance benefits as
+C<is_hashref>.
+
+There is also a family of functions with names like C<is_blessed_hashref>;
+these return true for blessed object instances that are implemented using
+the relevant underlying type.
+
 =item * Ignores overloading
 
 These functions ignore overloaded operators and simply check the
@@ -185,7 +229,7 @@ Support might be added, if a good reason arises.
 =back
 
 Additionally, two implementations are available, depending on the perl
-version you have. For perl that supports Custom OPs, we actually add
+version you have. For perls that supports Custom OPs, we actually add
 an OP (which is faster), and for perls that do not, we include
 an implementation that just calls an XS function - which is still
 faster than the Pure-Perl equivalent.
@@ -221,6 +265,17 @@ Check for a scalar reference.
     is_scalarref(\30);
     is_scalarref(\$value);
 
+Note that, even though a reference is itself a type of scalar value, a
+reference to another reference is not treated as a scalar reference:
+
+    !is_scalarref(\\1);
+
+The rationale for this is two-fold. First, callers that want to decide how
+to handle inputs based on their reference type will usually want to treat a
+ref-ref and a scalar-ref differently. Secondly, this more closely matches
+the behavior of the C<ref> builtin and of L<Scalar::Util/reftype>, which
+report a ref-ref as C<REF> rather than C<SCALAR>.
+
 =head2 is_arrayref($ref)
 
 Check for an array reference.
@@ -277,6 +332,114 @@ Check for a reference to a reference.
 
     is_refref( \[] ); # reference to array reference
 
+=head2 is_plain_scalarref($ref)
+
+Check for an unblessed scalar reference.
+
+    is_plain_scalarref(\"hello");
+    is_plain_scalarref(\30);
+    is_plain_scalarref(\$value);
+
+=head2 is_plain_ref($ref)
+
+Check for an unblessed reference to anything.
+
+    is_plain_ref([]);
+
+=head2 is_plain_arrayref($ref)
+
+Check for an unblessed array reference.
+
+    is_plain_arrayref([]);
+
+=head2 is_plain_hashref($ref)
+
+Check for an unblessed hash reference.
+
+    is_plain_hashref({});
+
+=head2 is_plain_coderef($ref)
+
+Check for an unblessed code reference.
+
+    is_plain_coderef( sub {} );
+
+=head2 is_plain_globref($ref)
+
+Check for an unblessed glob reference.
+
+    is_plain_globref( \*STDIN );
+
+=head2 is_plain_formatref($ref)
+
+Check for an unblessed format reference.
+
+    # set up format in STDOUT
+    format STDOUT =
+    .
+
+    # now we can test it
+    is_plain_formatref(bless *main::STDOUT{'FORMAT'} );
+
+=head2 is_plain_refref($ref)
+
+Check for an unblessed reference to a reference.
+
+    is_plain_refref( \[] ); # reference to array reference
+
+=head2 is_blessed_scalarref($ref)
+
+Check for a blessed scalar reference.
+
+    is_blessed_scalarref(bless \$value);
+
+=head2 is_blessed_ref($ref)
+
+Check for a blessed reference to anything.
+
+    is_blessed_ref(bless [], $class);
+
+=head2 is_blessed_arrayref($ref)
+
+Check for a blessed array reference.
+
+    is_blessed_arrayref(bless [], $class);
+
+=head2 is_blessed_hashref($ref)
+
+Check for a blessed hash reference.
+
+    is_blessed_hashref(bless {}, $class);
+
+=head2 is_blessed_coderef($ref)
+
+Check for a blessed code reference.
+
+    is_blessed_coderef( bless sub {}, $class );
+
+=head2 is_blessed_globref($ref)
+
+Check for a blessed glob reference.
+
+    is_blessed_globref( bless \*STDIN, $class );
+
+=head2 is_blessed_formatref($ref)
+
+Check for a blessed format reference.
+
+    # set up format for FH
+    format FH =
+    .
+
+    # now we can test it
+    is_blessed_formatref(bless *FH{'FORMAT'}, $class );
+
+=head2 is_blessed_refref($ref)
+
+Check for a blessed reference to a reference.
+
+    is_blessed_refref( bless \[], $class ); # reference to array reference
+
 =head1 SEE ALSO
 
 =over 4
@@ -315,4 +478,6 @@ The following people have been invaluable in their feedback and support.
 
 =item * p5pclub
 
+=item * Aaron Crane
+
 =back
